package com.wdkl.ncs.android.component.home.fragment

import android.media.AudioManager
import android.os.Build
import android.os.Handler
import android.os.Looper
import android.os.SystemClock
import android.util.Log
import android.view.View
import android.view.ViewGroup
import com.google.gson.Gson
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.settingconfig.SettingConfig
import com.wdkl.ncs.android.component.home.util.AppUtils
import com.wdkl.ncs.android.component.home.util.RingPlayHelper
import com.wdkl.ncs.android.component.home.util.Util
import com.wdkl.ncs.android.component.home.util.VoiceManagerUtil
import com.wdkl.ncs.android.component.nursehome.common.Constants
import com.wdkl.ncs.android.lib.base.BaseApplication
import com.wdkl.ncs.android.lib.utils.AppTool
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceChannel
import com.wdkl.ncs.android.middleware.tcp.channel.VideoUtil
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel
import com.wdkl.ncs.android.middleware.tcp.enums.TcpAction
import com.wdkl.ncs.android.middleware.utils.MessageEvent
import com.wdkl.ncs.janus.client.CallSessionCallback
import com.wdkl.ncs.janus.client.JanusClient
import com.wdkl.ncs.janus.client.VideoRoomCallback
import com.wdkl.ncs.janus.entity.Room
import com.wdkl.ncs.janus.rtc.WebRTCEngine
import com.wdkl.ncs.janus.util.EnumType
import com.wdkl.ncs.janus.util.JanusConstant
import kotlinx.android.synthetic.main.sky_voice_call_layout.*
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import org.webrtc.SurfaceViewRenderer
import java.math.BigInteger

class SkyCallFragment: BaseCallFragment(), CallSessionCallback {
    private val TAG = "SkyCallFragment"

    private var localSurfaceView: SurfaceViewRenderer? = null
    private var remoteSurfaceView: SurfaceViewRenderer? = null

    private val handler = Handler(Looper.getMainLooper())

    private var callEnded: Boolean = false

    private var outGoing: Boolean = false


    private var janusClient: JanusClient? = null
    private var room: Room?=null
    private var videoRoomCallback: VideoRoomCallback? = null

    private var callSuccess: Boolean = false

    override fun getLayId(): Int {
        if ("YUNPAI_H6".equals(Build.MODEL, true) || "rk3288".equals(Build.MODEL, true)
            || "KT10-3F".equals(Build.MODEL, true) || "KT10-2F".equals(Build.MODEL, true)
            || "KT8001-2F".equals(Build.MODEL, true)) {
            return R.layout.sky_voice_call_layout_land
        } else if ("m3520b_bnkj_zx".equals(Build.MODEL, true)) {
            return R.layout.sky_voice_call_layout_land_1920
        } else {
            return R.layout.sky_voice_call_layout
        }
    }

    override fun init() {
        //初始化计时器
        initCountDownTimer(sky_voice_call_timeout)

        //初始化 engine
        WebRTCEngine.getInstance().init(false, this.context)

        VoiceManagerUtil.setCallVoice(activity, SettingConfig.getExtensionCallVolume(activity))

        //初始化 janusClient
        janusClient = JanusClient(JanusConstant.JANUS_URL, Constants.sipId!!.toBigInteger())

        when (callState) {
            0 -> {
                //发起通话
                outGoing = true
                startOutgoing()
                RingPlayHelper.playRingTone(baseActivity, R.raw.ring_back2, true)

                janusClient!!.callState = EnumType.CallState.Outgoing
                room = Room(Constants.sipId!!.toBigInteger())
            }

            1 -> {
                //接受通话
            }
        }


        videoRoomCallback = VideoRoomCallback(janusClient, room, Constants.sipId!!.toBigInteger())
        videoRoomCallback!!.callSessionCallback = this
        janusClient!!.setJanusCallback(videoRoomCallback)
    }

    override fun bindEvent() {
        //通话挂断
        sky_voice_call_hangup.setOnClickListener {
            RingPlayHelper.stopRingTone()
            if (Constants.CALL_STATE == Constants.CALL_CALLING) {
                //结束sip通话
                Constants.CALL_STATE = Constants.CALL_STANDBY
                if (sky_voice_call_timer != null) {
                    sky_voice_call_timer.stop()
                }

                callEnd(true)
            } else {
                Constants.CALL_STATE = Constants.CALL_STANDBY
                DeviceChannel.calling = false
                VideoUtil.cancelVideoCall(Constants.deviceId, Constants.interactionId)
                cancelCall()
            }
        }
    }

    override fun destroy() {
        DeviceChannel.calling = false
        cancelTimer()
        Constants.CALL_STATE = Constants.CALL_STANDBY
        if (sky_voice_call_timer != null) {
            sky_voice_call_timer.stop()
        }
        RingPlayHelper.stopRingTone()
        AppUtils.setAudioMode(BaseApplication.appContext, AudioManager.MODE_NORMAL)
    }

    private fun startOutgoing() {
        callSuccess = false
        sky_voice_call_hangup.isEnabled = false
        VideoUtil.startVideoOutCall(Constants.deviceId)

        DeviceChannel.calling = true
        Constants.CALL_STATE = Constants.CALL_OUTGOING
        sky_voice_call_timeout.visibility = View.VISIBLE
        sky_voice_call_timer.visibility = View.GONE
        startTimer()

        AppTool.Time.delay(5000) {
            Log.d("tcp", "call success: $callSuccess")
            if (!callSuccess && !callEnded) {
                //呼叫失败
                showMessage("呼叫失败，服务器无响应或网络故障!")
                RingPlayHelper.stopRingTone()
                cancelCall()
            }
        }
    }

    //去电界面
    private fun showOutgoingCall() {
        Constants.CALL_STATE = Constants.CALL_OUTGOING
        sky_voice_call_calling_text.text = "正在呼叫，等待接听..."
        sky_voice_call_outgoing.visibility = View.VISIBLE
        sky_voice_call_incoming.visibility = View.GONE

        fullscreen_video_frame.visibility = View.VISIBLE
        pip_video_frame.visibility = View.VISIBLE
    }

    //呼叫取消
    private fun cancelCall() {
        cancelTimer()
        DeviceChannel.calling = false
        Constants.CALL_STATE = Constants.CALL_STANDBY
        callEnd(false)
    }

    private fun showCalling(audioOnly: Boolean) {
        if (audioOnly) {
            ll_voice_call.visibility = View.VISIBLE
        } else {
            //显示视频画面
            fullscreen_video_frame.visibility = View.VISIBLE
            pip_video_frame.visibility = View.VISIBLE
            ll_voice_call.visibility = View.GONE
        }

        cancelTimer()
        sky_voice_call_calling_text.text = "通话中..."
        sky_voice_call_timer.visibility = View.VISIBLE
        sky_voice_call_timer.base = SystemClock.elapsedRealtime()
        sky_voice_call_timer.start()

        if (Constants.hookOn) {
            //手柄放下，免提模式
            AppUtils.switchAudioMode(activity, true)
        } else {
            //手柄拿起，听筒模式
            AppUtils.switchAudioMode(activity, false)
        }
    }

    //通话结束
    override fun callEnd(handoff: Boolean) {
        synchronized(this) {
            if (callEnded) {
                return
            }
            callEnded = true
            DeviceChannel.calling = false
            Constants.CALL_STATE = Constants.CALL_STANDBY
            if (sky_voice_call_timer != null) {
                sky_voice_call_timer.stop()
            }

            Log.e(TAG, "call end !!!!!!!!!!!!!!!!!!")
            if (handoff) {
                VideoUtil.handoffVideoCall(Constants.deviceId, Constants.visitHostId, Constants.interactionId)
            }

            if (janusClient!!.webSocketChannel != null) {
                janusClient!!.callState = EnumType.CallState.Idle
                if (outGoing) {
                    janusClient!!.destroyRoom(janusClient!!.currentHandleId, null)
                } else {
                    janusClient!!.leaveRoom()
                }

                janusClient!!.setJanusCallback(null)
                janusClient!!.disConnect()
            }

            backToMain()
        }
    }


    /********************************************************
     ********************* webrtc通话回调 ********************
     * 注意: 如涉及到UI更新的需要在主线程处理，务必注意
     *******************************************************/
    override fun didChangeState(var1: EnumType.CallState?) {
        Log.e(TAG, "didChangeState: $var1")
        if (var1 == EnumType.CallState.Connected) {
            handler.post {
                RingPlayHelper.stopRingTone()
                //更新界面显示
                showCalling(false)
            }
        }
    }

    override fun didDisconnected(userId: String?) {
        Log.w(TAG, "didDisconnected: $userId")
        handler.post {
            showMessage("$userId 失去连接")
            callEnd(true)
        }
    }

    override fun didError(error: String?) {
        Log.e(TAG, "didError: $error")
        handler.post {
            showMessage("通话错误")
            callEnd(true)
        }
    }

    override fun didHangUp(handlerId: BigInteger?) {
        Log.e("hangup", "socket hangup")
        handler.post {
            callEnd(true)
        }
    }

    //处理本地视频画面
    override fun didCreateLocalVideoTrack() {
        Log.e(TAG, "didCreateLocalVideoTrack")
        handler.post {
            if (!callEnded) {
                if (localSurfaceView == null) {
                    val surfaceView = WebRTCEngine.getInstance().startPreview(true)
                    Log.e(TAG, "didCreateLocalVideoTrack surfaceView: $surfaceView")
                    if (surfaceView != null) {
                        localSurfaceView = surfaceView as SurfaceViewRenderer
                    } else {
                        callEnd(true)
                    }
                } else {
                    localSurfaceView!!.setZOrderMediaOverlay(true)
                }

                if (localSurfaceView!!.parent != null) {
                    (localSurfaceView!!.parent as ViewGroup).removeView(localSurfaceView)
                }

                if (fullscreen_video_frame != null) {
                    if (fullscreen_video_frame.getChildCount() != 0) {
                        fullscreen_video_frame.removeAllViews()
                    }
                    fullscreen_video_frame.addView(localSurfaceView)
                }
            }
        }
    }

    override fun didReceiveRemoteVideoTrack(userId: BigInteger?) {
        Log.e(TAG, "didReceiveRemoteVideoTrack==  userId: $userId, hostSip: ${Constants.hostSipId}")
        handler.post {
            if (!callEnded) {
                if (Constants.hostSipId!!.toBigInteger() != userId) {
                    //本地画面
                    Log.e(TAG, "didReceiveRemoteVideoTrack, local surfaceView = $localSurfaceView")
                    if (localSurfaceView != null) {
                        localSurfaceView!!.setZOrderMediaOverlay(true)
                        if (outGoing) {
                            if (localSurfaceView!!.parent != null) {
                                (localSurfaceView!!.parent as ViewGroup).removeView(localSurfaceView)
                            }
                            if (pip_video_frame != null) {
                                pip_video_frame.addView(localSurfaceView)
                            }
                        }
                    }

                    //远端画面
                    val surfaceView = WebRTCEngine.getInstance().setupRemoteVideo(userId, false)
                    Log.e(TAG, "didReceiveRemoteVideoTrack, remote surfaceView = $surfaceView")
                    if (surfaceView != null && fullscreen_video_frame != null) {
                        remoteSurfaceView = surfaceView as SurfaceViewRenderer
                        fullscreen_video_frame.removeAllViews()
                        if (remoteSurfaceView!!.parent != null) {
                            (remoteSurfaceView!!.parent as ViewGroup).removeView(remoteSurfaceView)
                        }
                        fullscreen_video_frame.addView(remoteSurfaceView)
                    }
                } else {
                    showMessage("主机已接听")
                }
            }
        }
    }

    override fun didCallEndWithReason(callEndReason: EnumType.CallEndReason?) {
        handler.post {
            callEnd(true)
        }
    }

    override fun didChangeMode(isAudioOnly: Boolean) {
        handler.post {
            //
        }
    }

    override fun didUserLeave(userId: BigInteger?) {
        Log.w(TAG, "didUserLeave: $userId")
        handler.post {
            callEnd(true)
        }
    }



    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMoonEvent(messageEvent: MessageEvent) {
        if (messageEvent.tag == Constants.VIDEO_MSG){
            val tcpModel = messageEvent.getMessage() as TcpModel
            var interactionVO: InteractionVO? = null
            if (tcpModel.data != null) {
                interactionVO = Gson().fromJson(tcpModel.data.toString(), InteractionVO::class.java)
            }
            Log.e(TAG, "收到tcp消息: " + tcpModel.getType() + " " + tcpModel.getAction())
            when (tcpModel.action){
                TcpAction.VideoAction.CALLING->{
                    //我方呼出，对方通话中
                    AppTool.Time.delay(3000) {
                        RingPlayHelper.stopRingTone()
                        cancelCall()
                        showMessage("对方忙线中!")
                    }
                }

                TcpAction.VideoAction.FAILED->{
                    //我方呼出，对方不在线，设备离线或其它错误
                    callSuccess = true
                    AppTool.Time.delay(3000) {
                        RingPlayHelper.stopRingTone()
                        cancelCall()
                        showMessage("呼叫失败，找不到设备或对方不在线!")
                    }
                }

                TcpAction.VideoAction.SUCCESS->{
                    callSuccess = true
                    DeviceChannel.calling = true
                    if (interactionVO != null) {
                        Constants.interactionId = interactionVO.id
                    }
                    Constants.visitHostId = tcpModel.fromId
                    Util.wakeUpAndUnlock(activity)
                    sky_voice_call_hangup.isEnabled = true
                    showOutgoingCall()
                    janusClient!!.connect()
                }

                TcpAction.VideoAction.ACCEPT-> {
                   //
                }

                //主机接听
                TcpAction.VideoAction.VIDEO_IN_CALL-> {
                    if (interactionVO != null) {
                        Constants.hostSipId = interactionVO.toSipId
                    }
                }

                TcpAction.VideoAction.VIDEO_ON -> {
                    //
                }

                TcpAction.VideoAction.REJECT->{
                    //我方呼出，对方拒绝
                    AppTool.Time.delay(1000) {
                        RingPlayHelper.stopRingTone()
                        cancelCall()
                        showMessage("对方拒绝!")
                    }
                }

                TcpAction.VideoAction.HANDOFF -> {
                    if (interactionVO != null && Constants.interactionId == interactionVO.id) {
                        callEnd(false)
                    }
                }
            }
        } else if (messageEvent.tag == Constants.EVENT_END_CALL) {
            if (messageEvent.getMessage() is String) {
                val str = messageEvent.getMessage() as String
                if (str.equals("cancel")) {
                    callEnd(false)
                } else {
                    callEnd(true)
                }
            }
        }
    }

}