package com.wdkl.ncs.android.middleware.api

import android.annotation.SuppressLint
import com.wdkl.ncs.android.lib.base.BaseApplication
import com.wdkl.ncs.android.lib.utils.UUID
import okhttp3.*

/**
 * @author  LDD
 * @Date   2018/1/11 下午6:12
 * @From   com.wdkl.ncs.android.lib.api
 * @Note   API管理器
 */
object ApiManager {

        /**
         * @Name  apiFactory
         * @Type  NetFectory
         * @Note  API构建者
         */
        @SuppressLint("StaticFieldLeak")
        private val apiFactory = NetFactory.build(BaseApplication.appContext)

        /**
         * @Name  com.wdkl.ncs.android.lib.api.ApiManager.Companion.BASE_URL
         * @Type  UrlManager
         * @Note  服务器URL控制器
         */
        public val urlManager = UrlManager.build()

        /** 全局分页条数 */
        val PAGE_SIZE = 10


        /**
         * @Name  com.wdkl.ncs.android.middleware.api.ApiManager.BASE_URL
         * @Type  BaseApi
         * @Note  基础API
         */
        val BASE_API :BaseApi by lazy { apiFactory.createService(BaseApi::class.java, urlManager.basic)}

        /**
         * @Name  com.wdkl.ncs.android.lib.api.ApiManager.Companion.API_AFTERSALE
         * @Type  AfterSaleApi
         * @Note  售后API
         */
        val API_AFTERSALE : AfterSaleApi by lazy { apiFactory.createService(AfterSaleApi::class.java, urlManager.device_url) }

        /**
         * @Name  com.wdkl.ncs.android.lib.api.ApiManager.Companion.API_MEMBER
         * @Type  MemberApi
         * @Note  会员API
         */
        val API_MEMBER: MemberApi by lazy { apiFactory.createService(MemberApi::class.java, urlManager.device_url) }


        /**
         * @Name  PASSPORT_API
         * @Type  PassportApi
         * @Note  护照
         */
        val PASSPORT_API :PassportApi by lazy { apiFactory.createService(PassportApi::class.java, urlManager.device_url) }

//   =========================================   以下是护士手表相关的  ==========================================================


        /**
         * @Name
         * @Type  API_RegisterDevice
         * @Note 获取设备信息和通讯地址
         */
        val API_WatchDeviceInfoAndTCPMailingAddress :WatchDeviceInfoAndTCPMailingAddressApi by lazy { apiFactory.createService(WatchDeviceInfoAndTCPMailingAddressApi::class.java, urlManager.device_url)}


        /**
         * 获取用户列表api
         */
        val API_WatchManageDevice :WatchManageDeviceApi by lazy { apiFactory.createService(WatchManageDeviceApi::class.java, urlManager.device_url) }

        /**
         * @Name  API_NurseHome
         * @Type  API_NurseHome
         * @Note  全部通话记录api
         */
        val API_WatchCallRecords :WatchCallRecordsApi by lazy { apiFactory.createService(WatchCallRecordsApi::class.java, urlManager.device_url)}

        /**
         * @Name
         * @Type  Api_WatchInitiator
         * @Note 获取指定用户呼叫记录
         */
        val Api_WatchNamedUser :WatchNamedUserApi by lazy { apiFactory.createService(WatchNamedUserApi::class.java, urlManager.device_url)}

        /**
         * @Name
         * @Type  Api_WatchUntreated
         * @Note
         */
        val Api_WatchUntreated :WatchUntreatedApi by lazy { apiFactory.createService(WatchUntreatedApi::class.java, urlManager.device_url)}

        /**
         * @author LDD
         * @From   ApiManager
         * @Date   2018/8/13 上午8:19
         * @Note   刷新Token
         * @param  refrehToken 刷新Token
         */
        fun refreshToken(refrehToken :String) : Response{
                val mediaType = MediaType.parse("text/x-markdown; charset=utf-8")
                val requestBody = "refrehToken"
                val okHttpClient = OkHttpClient()
                val getTokenRequest = Request.Builder()
                        .url("${urlManager.device_url}passport/token?refersh_token=$refrehToken")
                        .post(RequestBody.create(mediaType,requestBody))
                        .addHeader("uuid", UUID.uuid)
                        .build()
                val call = okHttpClient.newCall(getTokenRequest)
                return call.execute()
        }
}
