package com.wdkl.ncs.android.middleware.logic.presenter.home

import com.enation.javashop.net.engine.model.NetState
import com.enation.javashop.net.engine.plugin.connection.ConnectionQuality
import com.enation.javashop.net.engine.plugin.exception.ExceptionHandle
import com.enation.javashop.net.engine.utils.ThreadFromUtils
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import com.wdkl.ncs.android.lib.base.RxPresenter
import com.wdkl.ncs.android.lib.utils.ConnectionObserver
import com.wdkl.ncs.android.lib.utils.getEventCenter
import com.wdkl.ncs.android.lib.utils.getJsonString
import com.wdkl.ncs.android.lib.vo.NetStateEvent
import com.wdkl.ncs.android.middleware.api.WatchDeviceInfoAndTCPMailingAddressApi
import com.wdkl.ncs.android.middleware.api.WatchManageDeviceApi
import com.wdkl.ncs.android.middleware.di.MiddlewareDaggerComponent
import com.wdkl.ncs.android.middleware.logic.contract.home.HomeActivityContract
import com.wdkl.ncs.android.middleware.model.ServerIpInfo
import com.wdkl.ncs.android.middleware.model.dos.PartSettingDO
import com.wdkl.ncs.android.middleware.model.dto.TcpSeverDTO
import com.wdkl.ncs.android.middleware.model.vo.DeviceVO
import com.wdkl.ncs.android.middleware.model.vo.DeviceWatchInfoVO
import io.reactivex.disposables.Disposable
import javax.inject.Inject

class HomeActivityPresenter @Inject constructor(): RxPresenter<HomeActivityContract.View>(),HomeActivityContract.Presenter{


    /**
     *获取当前设备数据api
     */
    @Inject
    protected lateinit var registerDeviceApi: WatchDeviceInfoAndTCPMailingAddressApi

    @Inject
    protected lateinit var watchManageDeviceApi: WatchManageDeviceApi


    override fun bindDagger() {
        MiddlewareDaggerComponent.component.inject(this)
    }


    private val observer = object : ConnectionObserver<Any>(){
        override fun onStartWithConnection() {
            providerView().start()
        }

        override fun onNextWithConnection(result: Any, connectionQuality: ConnectionQuality) {
            providerView().complete()
            when (result) {
                is DeviceWatchInfoVO -> {
                    //设备信息
                    providerView().setDeviceDo(result)
                }

                is TcpSeverDTO -> {
                    //tcp服务器地址
                    providerView().setTcpServerHost(result)
                }

                is ServerIpInfo -> {
                    providerView().setServerInfo(result)
                }

                is PartSettingDO -> {
                    //设备设置数据
                    providerView().setDeviceSettingData(result)
                }

            }

        }

        override fun onErrorWithConnection(error: ExceptionHandle.ResponeThrowable, connectionQuality: ConnectionQuality) {
            providerView().onError(error.customMessage)
        }

        override fun attachSubscribe(var1: Disposable) {
            addDisposable(var1)
        }

        override fun onNoneNet() {
            getEventCenter().post(NetStateEvent(NetState.NONE))
        }
    }


    /**
     *获取当前设备数据
     */
    override fun getDeviceVO(ethMac:String) {

        registerDeviceApi.getDeviceMessage(ethMac).map {
            var gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
            var deviceVO = gson.fromJson(it.getJsonString(), DeviceWatchInfoVO::class.java)
            return@map deviceVO
        }
                .compose(ThreadFromUtils.defaultSchedulers())
                .subscribe(observer)

    }

    override fun getTcpServerHost() {
        registerDeviceApi.getTcpServerHost()
                .map {
                    val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                    val tcpServer = gson.fromJson(it.getJsonString(), TcpSeverDTO::class.java)

                    return@map tcpServer
                }
                .compose(ThreadFromUtils.defaultSchedulers())
                .subscribe(observer)
    }

    override fun getServerInfo() {
        registerDeviceApi.getServerInfo()
            .map {
                val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                val serverIpInfo = gson.fromJson(it.getJsonString(), ServerIpInfo::class.java)

                return@map serverIpInfo
            }
            .compose(ThreadFromUtils.defaultSchedulers())
            .subscribe(observer)
    }

    override fun getDeviceSettingData(partId: Int) {
        registerDeviceApi.getDeviceSettingData(partId).map {
            var partSettingDO = PartSettingDO()

            var gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
            partSettingDO = gson.fromJson(it.getJsonString(), PartSettingDO::class.java)
            return@map partSettingDO
        }
                .compose(ThreadFromUtils.defaultSchedulers())
                .subscribe(observer)
    }
}