package com.wdkl.ncs.android.middleware.logic.presenter.home

import com.enation.javashop.net.engine.plugin.connection.ConnectionQuality
import com.enation.javashop.net.engine.plugin.exception.ExceptionHandle
import com.enation.javashop.net.engine.utils.ThreadFromUtils
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import com.wdkl.ncs.android.lib.base.RxPresenter
import com.wdkl.ncs.android.lib.utils.ConnectionObserver
import com.wdkl.ncs.android.lib.utils.arrayObjects
import com.wdkl.ncs.android.lib.utils.toJsonObject
import com.wdkl.ncs.android.lib.utils.valueJsonArray
import com.wdkl.ncs.android.middleware.api.WatchManageDeviceApi
import com.wdkl.ncs.android.middleware.di.MiddlewareDaggerComponent
import com.wdkl.ncs.android.middleware.logic.contract.home.WatchActivityContract
import com.wdkl.ncs.android.middleware.model.vo.WatchContactsVO
import io.reactivex.disposables.Disposable
import javax.inject.Inject

class WatchActivityPresenter @Inject constructor(): RxPresenter<WatchActivityContract.View>(),WatchActivityContract.Presenter{

    @Inject
    protected lateinit var deviceApi: WatchManageDeviceApi

    override fun bindDagger() {
        MiddlewareDaggerComponent.component.inject(this)
    }

    private val observer = object : ConnectionObserver<Any>(){
        override fun onStartWithConnection() {
            providerView().start()
        }

        override fun onNextWithConnection(result: Any, connectionQuality: ConnectionQuality) {
            providerView().complete("")
            providerView().render(result as ArrayList<WatchContactsVO>)
        }

        override fun onErrorWithConnection(error: ExceptionHandle.ResponeThrowable, connectionQuality: ConnectionQuality) {
            providerView().onError(error.customMessage)
        }

        override fun attachSubscribe(var1: Disposable) {
            addDisposable(var1)
        }
    }

    override fun loadData(page: Int,deviceId: Int) {
        deviceApi.getWatchContacts(page,deviceId)
                .map {
                    var data = ArrayList<WatchContactsVO>()
                    var gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                    it.toJsonObject().valueJsonArray("data").arrayObjects().forEach({ dic ->
                        data.add(gson.fromJson(dic.toString(),WatchContactsVO::class.java))
                    })
                    return@map data
                }
                .compose(ThreadFromUtils.defaultSchedulers())
                .subscribe(observer)
    }
}