package com.wdkl.ncs.android.middleware.logic.presenter.home

import com.enation.javashop.net.engine.plugin.connection.ConnectionQuality
import com.enation.javashop.net.engine.plugin.exception.ExceptionHandle
import com.enation.javashop.net.engine.utils.ThreadFromUtils
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import com.wdkl.ncs.android.lib.base.RxPresenter
import com.wdkl.ncs.android.lib.utils.*
import com.wdkl.ncs.android.middleware.api.WatchDeviceInfoAndTCPMailingAddressApi
import com.wdkl.ncs.android.middleware.di.MiddlewareDaggerComponent
import com.wdkl.ncs.android.middleware.logic.contract.home.WatchDeviceContract
import com.wdkl.ncs.android.middleware.model.dos.DeviceDO
import com.wdkl.ncs.android.middleware.model.dto.TcpSeverDTO
import com.wdkl.ncs.android.middleware.model.vo.DeviceVO
import io.reactivex.disposables.Disposable
import javax.inject.Inject

/**
 * RegisterActivity逻辑控制
 */
class WatchDevicePresenter@Inject constructor() :RxPresenter<WatchDeviceContract.View>(),WatchDeviceContract.Presenter {


    /**
     *获取当前设备数据api
     */
    @Inject
    protected lateinit var registerDeviceApi: WatchDeviceInfoAndTCPMailingAddressApi

    /**
     *依赖注入初始化
     */
    override fun bindDagger() {
        MiddlewareDaggerComponent.component.inject(this)
    }

    /**
     * 数据监听者
     */
    private val observable = object: ConnectionObserver<Any>(){
        override fun onStartWithConnection() {
            providerView()
        }

        override fun onNextWithConnection(result: Any, connectionQuality: ConnectionQuality) {
            when (result) {
                is DeviceVO -> {
                    providerView().complete()
                    providerView().showData(result as DeviceVO)
                }

                is TcpSeverDTO -> {
                    //tcp服务器地址
                    providerView().complete("")
                    providerView().setTcpServerHost(result)
                }
            }

        }

        override fun onErrorWithConnection(error: ExceptionHandle.ResponeThrowable, connectionQuality: ConnectionQuality) {
            providerView().onError(error.customMessage)
        }

        override fun attachSubscribe(var1: Disposable) {
        addDisposable(var1)
        }

    }

    /**
     *获取当前设备数据
     */
    override fun loadData(ethMac:String) {

        registerDeviceApi.getDeviceMessage(ethMac).map {
            var deviceVO = DeviceVO()

            var gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
            deviceVO = gson.fromJson(it.getJsonString(), DeviceVO::class.java)
          return@map deviceVO
        }
                .compose(ThreadFromUtils.defaultSchedulers())
                .subscribe(observable)

    }

    override fun loadTcpServerHost() {
        registerDeviceApi.getTcpServerHost()
                .map {
                    val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                    val tcpServer = gson.fromJson(it.getJsonString(), TcpSeverDTO::class.java)

                    return@map tcpServer
                }
                .compose(ThreadFromUtils.defaultSchedulers())
                .subscribe(observable)
    }
}